import apt_pkg
import codecs
import gettext
import gi
import os

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk

from mintreport import InfoReport, InfoReportAction

class LanguagePack:

    def __init__(self, category, language, dependency, package):
        self.category = category
        self.language = language
        self.dependency = dependency
        self.package = package

class Report(InfoReport):

    def __init__(self):

        gettext.install("mintreport", "/usr/share/locale", names="ngettext")

        self.title = _("Install language packs")
        self.icon = "xsi-flag-symbolic"
        self.has_ignore_button = True

    def is_pertinent(self):
        # Defines whether this report should show up
        language = os.getenv('LANG')
        if language is None:
            return False
        else:
            return self.check_missing_packages(language)

    def get_descriptions(self):
        # Return the descriptions
        descriptions = []
        descriptions.append(_("The following localization packages are missing to properly support your language:"))
        descriptions.append("\n".join(self.missing_pack_names))
        return descriptions

    def get_actions(self):
        # Return available actions
        actions = []
        action = InfoReportAction(label=_("Install the Language Packs"), callback=self.callback)
        action.set_style(Gtk.STYLE_CLASS_SUGGESTED_ACTION)
        actions.append(action)
        return actions

    def callback(self, data):
        self.install_packages(self.missing_pack_names)
        # reload
        return True

    def check_missing_packages(self, locale):
        self.missing_pack_names = []
        apt_pkg.init()
        self.cache = apt_pkg.Cache(None)

        (language_code, country_code) = self.split_locale(locale)

        self.language_packs = []
        with codecs.open("/usr/share/linuxmint/mintlocale/language_packs", 'r', encoding='utf-8') as f:
            for line in f:
                line = line.strip()
                columns = line.split(":")
                if len(columns) == 4:
                    (category, language, dependency, package) = columns
                    if package.endswith("-"):
                        self.language_packs.append(LanguagePack(category, language, dependency, "%sLANG" % package))
                        self.language_packs.append(LanguagePack(category, language, dependency, "%sLANG-COUNTRY" % package))
                    else:
                        self.language_packs.append(LanguagePack(category, language, dependency, package))

        # Check if the language packs are installed
        for language_pack in self.language_packs:
            if language_pack.language == "" or language_pack.language == language_code:
                pkgname = language_pack.package.replace("LANG", language_code).replace("COUNTRY", country_code)
                depname = language_pack.dependency
                if pkgname in self.cache:
                    pkg = self.cache[pkgname]
                    if (pkg.has_versions and pkg.current_state != apt_pkg.CURSTATE_INSTALLED):
                        if depname != "":
                            if depname in self.cache and self.cache[depname].current_state == apt_pkg.CURSTATE_INSTALLED:
                                if pkgname not in self.missing_pack_names:
                                    self.missing_pack_names.append(pkgname)
                        else:
                            if pkgname not in self.missing_pack_names:
                                self.missing_pack_names.append(pkgname)

        if len(self.missing_pack_names) == 0:
            return False
        else:
            return True

    def split_locale(self, locale_code):
        if "_" in locale_code:
            split = locale_code.split("_")
            language_code = split[0]
            language = language_code
            country_code = split[1].lower().split('@')[0].strip()
            country = country_code
        else:
            language_code = locale_code
            country_code = ""

        return (language_code, country_code)

if __name__ == "__main__":
    report = Report()
    print(report.is_pertinent())



#!/usr/bin/python3
import apt
import argparse
import datetime
import gi
import json
import os
import platform
import setproctitle
import shutil
import subprocess
import sys
import threading
import xapp.SettingsWidgets as Xs
import xapp.threading as xt
import xapp.util

gi.require_version("Gtk", "3.0")
gi.require_version('GtkSource', '3.0')
gi.require_version('XApp', '1.0')
from gi.repository import Gtk, Gdk, GtkSource, Gio

from common import InfoReportContainer, DATA_DIR, INFO_DIR, prefix_version, read_dmi, clean_brand
from bios import BIOSListWidget
from pci import PCIListWidget
from usb import USBListWidget
from gpu import GPUListWidget

setproctitle.setproctitle("mintreport")
_ = xapp.util.l10n("mintreport")

TMP_DIR = "/tmp/mintreport"
TMP_INFO_DIR = os.path.join(TMP_DIR, "reports")
TMP_INXI_FILE = os.path.join(TMP_DIR, "inxi")

UNPACK_DIR = os.path.join(TMP_DIR, "crash")
CRASH_ARCHIVE = os.path.join(TMP_DIR, "crash.tar.gz")

COL_CRASH_DAY, COL_CRASH_DATE, COL_CRASH_TIME, COL_CRASH_TIMEZONE, COL_CRASH_PID, COL_CRASH_EXE, COL_CRASH_OBJECT = range(7)

COL_INFO_ICON, COL_INFO_TITLE, COL_INFO_REPORT = range(3)

class CrashReport():

    def __init__(self, timestamp, pid, sig, executable):
        self.timestamp = timestamp
        self.pid = pid
        self.sig = sig
        self.executable = executable

class MyApplication(Gtk.Application):
    # Main initialization routine
    def __init__(self, application_id, flags):
        Gtk.Application.__init__(self, application_id=application_id, flags=flags)
        self.connect("activate", self.activate)
        self.connect("command-line", self.on_command_line)
        self.report_window = None
        self.page = None

    def on_command_line(self, app, command_line):
        # Parse fresh args each time
        parser = argparse.ArgumentParser(add_help=False)
        group = parser.add_mutually_exclusive_group()
        group.add_argument("--reports", action="store_const", dest="page", const="reports")
        group.add_argument("--crashes", action="store_const", dest="page", const="crashes")
        group.add_argument("--usb", action="store_const", dest="page", const="usb")
        group.add_argument("--info", action="store_const", dest="page", const="info")
        group.add_argument("--bios", action="store_const", dest="page", const="bios")
        group.add_argument("--pci", action="store_const", dest="page", const="pci")
        group.add_argument("--gpu", action="store_const", dest="page", const="gpu")
        argv = command_line.get_arguments()[1:]
        args, _ = parser.parse_known_args(argv)
        if args.page is None:
            args.page = "info"
        self.page = args.page
        self.activate(app)
        return 0

    def activate(self, application):
        if self.report_window is None:
            self.report_window = MintReportWindow(self)
            self.add_window(self.report_window.window)
        window = self.report_window.window
        window = self.get_windows()[0]
        window.present()
        window.show_all()
        self.report_window.show_page(self.page)

def kill_process(process):
    process.kill()

def get_process_output(command):
    timeout = 2.0  # Timeout for any subprocess before aborting it

    lines = []
    p = subprocess.Popen(command, stdout=subprocess.PIPE)
    timer = threading.Timer(timeout, kill_process, [p])
    timer.start()
    while True:
        line = p.stdout.readline()
        if not line:
            break
        if line != '':
            lines.append(line.decode('utf-8'))
    timer.cancel()
    return lines


def get_graphic_cards():
    cards = {}
    count = 0
    envpath = os.environ["PATH"]
    os.environ["PATH"] = envpath + ":/usr/local/sbin:/usr/sbin:/sbin"
    for card in get_process_output("lspci"):
        for prefix in ["VGA compatible controller:", "3D controller:", "Display controller:"]:
            if prefix in card:
                cardName = card.split(prefix)[1].split("(rev")[0].strip()
                cards[count] = cardName
                count += 1
    os.environ["PATH"] = envpath
    return cards


def get_disk_size():
    disksize = 0
    try:
        out = get_process_output(("lsblk", "--json", "--output", "size", "--bytes", "--nodeps"))
        jsonobj = json.loads(''.join(out))
    except Exception:
        return _("Unknown size"), False

    for blk in jsonobj['blockdevices']:
        disksize += int(blk['size'])

    return disksize, (len(jsonobj['blockdevices']) > 1)


def get_proc_infos():
    # For some platforms, 'model name' will no longer take effect.
    # We can try our best to detect it, but if all attempts failed just leave it to be "Unknown".
    # Source: https://github.com/dylanaraps/neofetch/blob/6dd85d67fc0d4ede9248f2df31b2cd554cca6c2f/neofetch#L2163
    cpudetect = ("model name", "Hardware", "Processor", "cpu model", "chip type", "cpu type")
    infos = [
        ("/proc/cpuinfo", [("cpu_name", cpudetect), ("cpu_siblings", ("siblings",)), ("cpu_cores", ("cpu cores",))]),
        ("/proc/meminfo", [("mem_total", ("MemTotal",))])
    ]
    result = {}
    for (proc, pairs) in infos:
        for line in get_process_output(("cat", proc)):
            for (key, start) in pairs:
                for item in start:
                    if line.startswith(item):
                        result[key] = line.split(':', 1)[1].strip()
                        break
    if "cpu_name" not in result:
        result["cpu_name"] = _("Unknown CPU")
    if "mem_total" not in result:
        result["mem_total"] = _("Unknown size")
    return result

class MintReportWindow():

    def __init__(self, application):

        self.application = application
        self.settings = Gio.Settings(schema_id="com.linuxmint.report")

        os.system("mkdir -p %s" % TMP_DIR)
        os.system("rm -rf %s/*" % TMP_DIR)
        os.system("mkdir -p %s" % UNPACK_DIR)
        os.system("cp -R %s/* %s/" % (DATA_DIR, TMP_DIR))

        # Create a CSS provider
        css_provider = Gtk.CssProvider()
        css_provider.load_from_data(b"""
            .logo-box {
                background-color: #86be43;
                border-radius: 10px;
                padding: 12px;
            }
        """)
        screen = Gdk.Screen.get_default()
        Gtk.StyleContext.add_provider_for_screen(
            screen,
            css_provider,
            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
        )

        self._cache = None
        # Set the Glade file
        gladefile = "/usr/share/linuxmint/mintreport/mintreport.ui"
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain("mintreport")
        self.builder.add_from_file(gladefile)
        self.window = self.builder.get_object("main_window")
        self.window.set_title(_("System Information"))
        self.window.set_icon_name("mintreport")

        self.stack = self.builder.get_object("main_stack")
        self.spinner = self.builder.get_object("crash_spinner")

        # the crashes treeview
        self.treeview_crashes = self.builder.get_object("treeview_crashes")

        def render_date(column, cell, model, i, *args):
            cell.props.text = " ".join([model[i][COL_CRASH_DAY],
                                        model[i][COL_CRASH_DATE],
                                        model[i][COL_CRASH_TIME],
                                        model[i][COL_CRASH_TIMEZONE]])

        def sort_by_date(model, a, b, *args):
            date_a = datetime.date.from_iso_format(model[a][COL_CRASH_DATE])
            date_b = datetime.date.from_iso_format(model[b][COL_CRASH_DATE])

            if date_a < date_b:
                return -1
            elif date_a > date_b:
                return 1

            time_a = datetime.time.fromisoformat(model[a][COL_CRASH_TIME])
            time_b = datetime.time.fromisoformat(model[b][COL_CRASH_TIME])

            if time_a < time_b:
                return -1
            elif time_a > time_b:
                return 1

            return 0

        cell_renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn('', cell_renderer)
        column.set_cell_data_func(cell_renderer, render_date)
        column.set_resizable(True)
        cell_renderer.set_property("ypad", 6)
        self.treeview_crashes.append_column(column)
        column = Gtk.TreeViewColumn("", Gtk.CellRendererText(), text=COL_CRASH_PID)
        column.set_sort_column_id(COL_CRASH_PID)
        column.set_resizable(True)
        self.treeview_crashes.append_column(column)
        column = Gtk.TreeViewColumn("", Gtk.CellRendererText(), text=COL_CRASH_EXE)
        column.set_sort_column_id(COL_CRASH_EXE)
        column.set_resizable(True)
        self.treeview_crashes.append_column(column)
        self.treeview_crashes.show()
        self.model_crashes = Gtk.ListStore(str, str, str, str, str, str, object) # timestamp, pid, exe, object
        self.model_crashes.set_sort_func(COL_CRASH_DATE, sort_by_date)
        self.treeview_crashes.set_model(self.model_crashes)

        self.buffer = GtkSource.Buffer()
        self.language_manager = GtkSource.LanguageManager()
        style_manager = GtkSource.StyleSchemeManager()
        self.buffer.set_style_scheme(style_manager.get_scheme("oblivion"))
        self.sourceview = GtkSource.View.new_with_buffer(self.buffer)
        self.builder.get_object("scrolledwindow_crash").add(self.sourceview)
        self.sourceview.show()

        self.treeview_crashes.get_selection().connect("changed", self.on_crash_selected)

        self.bugtracker = "https://bugs.launchpad.net/"

        self.localfiles_button = self.builder.get_object("button_browse_crash_report")
        self.bugtracker_button = self.builder.get_object("button_open_bugtracker")
        self.pastebin_button = self.builder.get_object("button_pastebin")
        self.localfiles_button.connect("clicked", self.on_button_browse_crash_report_clicked)
        self.bugtracker_button.connect("clicked", self.on_button_open_bugtracker_clicked)
        self.pastebin_button.connect("clicked", self.on_button_pastebin_clicked)

        # the info treeview
        self.info_button_box = self.builder.get_object("info_button_box")
        self.info_descriptions_box = self.builder.get_object("info_descriptions_box")

        self.treeview_info = self.builder.get_object("treeview_info")
        renderer = Gtk.CellRendererPixbuf()
        column = Gtk.TreeViewColumn("", renderer, icon_name=COL_INFO_ICON)
        self.treeview_info.append_column(column)

        renderer = Gtk.CellRendererText()
        renderer.set_property("ypad", 6)
        column = Gtk.TreeViewColumn("", renderer, text=COL_INFO_TITLE)
        column.set_sort_column_id(COL_INFO_TITLE)
        column.set_resizable(True)
        self.treeview_info.append_column(column)
        self.treeview_info.show()
        self.model_info = Gtk.TreeStore(str, str, object) # icon, name, report
        self.model_info.set_sort_column_id(COL_INFO_TITLE, Gtk.SortType.ASCENDING)
        self.treeview_info.set_model(self.model_info)

        self.treeview_info.get_selection().connect("changed", self.on_info_selected)

        if os.path.exists("/usr/bin/coredumpctl"):
            self.builder.get_object("crash_internal_stack").set_visible_child_name("page_reports")
            self.load_crashes()
        else:
            self.builder.get_object("crash_internal_stack").set_visible_child_name("page_error")

        page = Xs.SettingsPage()
        page.set_spacing(24)
        page.set_margin_left(0)
        page.set_margin_right(0)
        page.set_margin_top(0)
        page.set_margin_bottom(0)
        self.builder.get_object("box_info_widget").add(page)

        self.sysinfo_section = page.add_section()

        accel_group = Gtk.AccelGroup()
        self.window.add_accel_group(accel_group)

        # Menubar
        menu = self.builder.get_object("main_menu")

        refresh_button = self.builder.get_object("refresh_button")
        refresh_button.connect('clicked', self.on_menu_refresh)
        key, mod = Gtk.accelerator_parse("<Control>R")
        refresh_button.add_accelerator("activate", accel_group, key, mod, Gtk.AccelFlags.VISIBLE)

        item = Gtk.ImageMenuItem(label=_("Restore ignored reports"))
        item.connect('activate', self.on_menu_restore)
        menu.append(item)
        menu.append(Gtk.SeparatorMenuItem())

        item = Gtk.ImageMenuItem(label=_("Quit"))
        image = Gtk.Image.new_from_icon_name("xsi-exit-symbolic", Gtk.IconSize.MENU)
        item.set_image(image)
        item.connect('activate', self.on_menu_quit)
        key, mod = Gtk.accelerator_parse("<Control>Q")
        item.add_accelerator("activate", accel_group, key, mod, Gtk.AccelFlags.VISIBLE)
        key, mod = Gtk.accelerator_parse("<Control>W")
        item.add_accelerator("activate", accel_group, key, mod, Gtk.AccelFlags.VISIBLE)
        menu.append(item)

        item = Gtk.ImageMenuItem()
        item.set_image(Gtk.Image.new_from_icon_name("xsi-help-about-symbolic", Gtk.IconSize.MENU))
        item.set_label(_("About"))
        item.connect("activate", self.open_about)
        menu.append(item)

        menu.show_all()

        # Info page
        procInfos = get_proc_infos()
        infos = []

         # Product
        version = prefix_version(read_dmi('product_version'))
        vendor = clean_brand(read_dmi('sys_vendor'))
        sku = read_dmi('product_sku')
        product = f"{vendor} {read_dmi('product_name')} {version} {sku}"
        product = " ".join(product.split()) # remove unecessary spaces
        infos.append((_("Computer"), product))

        try:
            (memsize, memunit) = procInfos['mem_total'].split(" ")
            memsize = float(memsize)
        except ValueError:
            memsize = procInfos['mem_total']
            memunit = ""
        processorName = procInfos['cpu_name'].replace("(R)", "\u00A9").replace("(TM)", "\u2122")
        if 'cpu_cores' in procInfos:
            processorName = processorName + " \u00D7 " + procInfos['cpu_cores']

        with open("/etc/linuxmint/info") as f:
            config = dict([line.strip().split("=") for line in f])
        distribution = "Linux Mint"
        if os.path.exists("/usr/share/doc/debian-system-adjustments/copyright"):
            distribution = "LMDE"
        release = config['RELEASE']
        edition = config['EDITION'].replace('"', '')
        architecture = "64-bit"
        if platform.machine() not in ["x86_64", "aarch64"]:
            architecture = "32-bit"
        infos.append((_("Operating System"), f"{distribution} {release} - {edition} {architecture}"))
        infos.append((_("Linux Kernel"), platform.release()))
        infos.append((_("Processor"), processorName))
        cards = get_graphic_cards()
        for card in cards:
            infos.append((_("Graphics Card"), cards[card]))
        if memunit == "kB":
            infos.append((_("Memory"), '%.1f %s' % ((float(memsize)/(1000**2)), _("GB"))))
        else:
            infos.append((_("Memory"), procInfos['mem_total']))

        diskSize, multipleDisks = get_disk_size()
        if multipleDisks:
            diskText = _("Hard Drives")
        else:
            diskText = _("Hard Drive")
        try:
            infos.append((diskText, '%.1f %s' % ((diskSize / (1000**3)), _("GB"))))
        except:
            infos.append((diskText, diskSize))

        desktop = os.getenv("XDG_CURRENT_DESKTOP")
        if "cinnamon" in desktop.lower():
            desktop = f"Cinnamon {os.getenv("CINNAMON_VERSION")}"
        infos.append((_("Desktop Environment"), desktop))

        display_server_name = "X11"

        if os.getenv("XDG_SESSION_TYPE") == "wayland":
            display_server_name = "Wayland"

        infos.append((_("Display Server"), display_server_name))

        for (key, value) in infos:
            self.sysinfo_section.add_row(self.create_info_row(key, value))

        self.inxi_info = ""
        self.builder.get_object("button_sysinfo_copy").connect("clicked", self.copy_inxi_info)
        self.builder.get_object("button_sysinfo_upload").connect("clicked", self.upload_inxi_info)


        # USB page
        self.usb_widget = USBListWidget()
        self.builder.get_object("box_usb_widget").pack_start(self.usb_widget, True, True, 0)

        # PCI page
        self.pci_widget = PCIListWidget()
        self.builder.get_object("box_pci_widget").pack_start(self.pci_widget, True, True, 0)

        # BIOS page
        self.bios_widget = BIOSListWidget()
        self.builder.get_object("box_bios_widget").add(self.bios_widget)

        self.gpu_widget = GPUListWidget()
        self.builder.get_object("box_gpu_widget").add(self.gpu_widget)

        self.load_inxi_info()
        self.load_reports()
        self.load_usb()
        self.load_pci()
        self.load_bios()
        self.load_gpu()

    def show_page(self, page_name):
        page_name = f"page_{page_name}"
        self.stack.set_visible_child_name(page_name)

    def create_info_row(self, key, value):
        widget = Xs.SettingsWidget()
        widget.set_spacing(40)
        labelKey = Gtk.Label.new(key)
        widget.pack_start(labelKey, False, False, 0)
        labelKey.get_style_context().add_class("dim-label")
        labelValue = Gtk.Label.new(value)
        labelValue.set_selectable(True)
        labelValue.set_line_wrap(True)
        widget.pack_end(labelValue, False, False, 0)
        return widget

    @property
    def cache(self):
        """Cache data from apt"""
        if self._cache is None:
            # Lazily initialize cache data because it is only used when a crash dump is selected
            # on the 'Crash reports' page
            self._cache = apt.Cache()

        return self._cache

    def open_about(self, widget):
        dlg = Gtk.AboutDialog()
        dlg.set_transient_for(self.window)
        dlg.set_title(_("About"))
        dlg.set_program_name("mintreport")
        dlg.set_comments(_("System Information"))
        try:
            h = open('/usr/share/common-licenses/GPL', encoding="utf-8")
            s = h.readlines()
            gpl = ""
            for line in s:
                gpl += line
            h.close()
            dlg.set_license(gpl)
        except Exception as e:
            print (e)

        dlg.set_version("__DEB_VERSION__")
        dlg.set_icon_name("mintreport")
        dlg.set_logo_icon_name("mintreport")
        dlg.set_website("https://www.github.com/linuxmint/mintreport")
        def close(w, res):
            if res == Gtk.ResponseType.CANCEL or res == Gtk.ResponseType.DELETE_EVENT:
                w.destroy()
        dlg.connect("response", close)
        dlg.show()

    def on_menu_restore(self, widget):
        self.settings.reset("ignored-reports")
        self.load_reports()

    def on_menu_refresh(self, widget):
        self.load_reports()
        self.load_crashes()

    def on_menu_quit(self, widget):
        self.application.quit()

    @xt.run_async
    def load_inxi_info(self):
        try:
            sysinfo = subprocess.check_output("LANG=C inxi -Fxxxrzc0 --usb", shell=True).decode("utf-8", errors='replace')
            self.inxi_info = sysinfo
            self.enable_inxi_buttons()
            with open(TMP_INXI_FILE, "w") as f:
                f.write(sysinfo)
        except Exception as e:
            subprocess.Popen(['notify-send', '-i', 'xsi-dialog-error-symbolic', _("An error occurred while gathering the system information."), str(e)])
            print (e)

    @xt.run_idle
    def enable_inxi_buttons(self):
        self.builder.get_object("button_sysinfo_copy").set_sensitive(True)
        self.builder.get_object("button_sysinfo_upload").set_sensitive(True)

    def copy_inxi_info(self, button):
        clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
        clipboard.set_text("[code]\n%s[/code]\n" % self.inxi_info, -1)
        subprocess.Popen(['notify-send', '-i', 'xsi-dialog-information-symbolic', _("System information copied"), _("Your system information was copied into your clipboard so you can paste it on the forums.")])

    def upload_inxi_info(self, button):
        try:
            output = subprocess.check_output("pastebin %s" % TMP_INXI_FILE, shell=True).decode("UTF-8")
            link = output.rstrip('\x00').strip() # Remove ASCII null termination with \x00
            clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
            clipboard.set_text(link, -1)
            subprocess.Popen(['notify-send', '-i', 'xsi-dialog-information-symbolic', _("System information uploaded"), _("Your system information was uploaded to %s. This link was placed in your clipboard.") % link])
        except Exception as e:
            subprocess.Popen(['notify-send', '-i', 'xsi-dialog-error-symbolic', _("An error occurred while uploading the system information"), _("Copy and paste the system information manually into a pastebin site like https://pastebin.com."), str(e)])

    @xt.run_idle
    def add_report_to_treeview(self, report):
        iter = self.model_info.insert_before(None, None)
        self.model_info.set_value(iter, COL_INFO_ICON, report.instance.icon)
        self.model_info.set_value(iter, COL_INFO_TITLE, report.instance.title)
        self.model_info.set_value(iter, COL_INFO_REPORT, report)

    @xt.run_idle
    def clear_info_treeview(self):
        self.model_info.clear()
        self.builder.get_object("info_box").hide()

    @xt.run_idle
    def show_info_spinner(self):
        self.builder.get_object("box_info_stack").set_visible_child_name("spinner")
        self.builder.get_object("info_spinner").start()

    @xt.run_idle
    def show_info_reports(self):
        if self.num_info_found > 0:
            self.builder.get_object("box_info_stack").set_visible_child_name("reports")
            iter_first = self.model_info.get_iter_first()
            if iter_first is not None:
                self.treeview_info.get_selection().select_iter(iter_first)
        else:
            self.builder.get_object("box_info_stack").set_visible_child_name("done")
        self.builder.get_object("info_spinner").stop()

    @xt.run_async
    def load_reports(self):
        self.loading = True
        self.clear_info_treeview()
        self.show_info_spinner()
        self.num_info_found = 0
        if os.path.exists(INFO_DIR):
            ignored_paths = self.settings.get_strv("ignored-reports")
            for dir_name in sorted(os.listdir(INFO_DIR)):
                path = os.path.join(INFO_DIR, dir_name)
                uuid = dir_name.split("_")[-1]
                print ("Checking %s" % dir_name)
                if uuid not in ignored_paths:
                    try:
                        report = InfoReportContainer(uuid, path)
                        if report.instance.is_pertinent():
                            self.add_report_to_treeview(report)
                            self.num_info_found += 1
                    except Exception as e:
                        print("Failed to load report %s: \n%s\n" % (dir_name, e))
        self.loading = False
        self.show_info_reports()

    @xt.run_async
    def load_usb(self):
        self.usb_widget.load()

    @xt.run_async
    def load_pci(self):
        self.pci_widget.load()

    @xt.run_async
    def load_bios(self):
        self.bios_widget.load()

    @xt.run_async
    def load_gpu(self):
        self.gpu_widget.load()

    def on_info_selected(self, selection):
        if self.loading:
            return
        model, iter = selection.get_selected()
        if iter is not None:
            report = model.get_value(iter, COL_INFO_REPORT)
            descriptions = report.instance.get_descriptions()
            actions = report.instance.get_actions()
            self.builder.get_object("info_icon_image").set_from_icon_name(report.instance.icon, Gtk.IconSize.DIALOG)
            self.builder.get_object("info_title_label").set_text(report.instance.title)
            for child in self.info_descriptions_box.get_children():
                self.info_descriptions_box.remove(child)
            for description in descriptions:
                label = Gtk.Label(label=description)
                label.set_use_markup(True)
                label.set_line_wrap(True)
                self.info_descriptions_box.add(label)
            for child in self.info_button_box.get_children():
                self.info_button_box.remove(child)
            for action in actions:
                button = Gtk.Button(label=action.label)
                button.connect("clicked", self.on_info_action_clicked, action.callback, action.data)
                if action.style is not None:
                    button.get_style_context().add_class(action.style)
                self.info_button_box.add(button)
            if report.instance.has_ignore_button:
                button = Gtk.Button(label=_("Ignore this report"))
                button.connect("clicked", self.on_ignore_button_clicked)
                self.info_button_box.add(button)
            self.builder.get_object("info_box").show_all()

    def on_info_action_clicked(self, button, callback, data):
        self.window.set_sensitive(False)
        reload_requested = callback(data)
        if reload_requested:
            self.load_reports()
        self.window.set_sensitive(True)

    def on_ignore_button_clicked(self, button):
            model, iter = self.treeview_info.get_selection().get_selected()
            if iter is not None:
                report = model.get_value(iter, COL_INFO_REPORT)
                dialog = Gtk.MessageDialog(self.window, 0, Gtk.MessageType.QUESTION, Gtk.ButtonsType.OK_CANCEL, _("Are you sure you want to ignore this report?"))
                dialog.format_secondary_text(_("The report will be permanently ignored and will no longer be visible."))
                response = dialog.run()
                dialog.destroy()
                if response == Gtk.ResponseType.OK:
                    ignored_uuids = self.settings.get_strv("ignored-reports")
                    if report.uuid not in ignored_uuids:
                        ignored_uuids.append(report.uuid)
                        self.settings.set_strv("ignored-reports", ignored_uuids)
                        self.num_info_found -= 1
                        model.remove(iter)
                        if self.num_info_found == 0:
                            self.show_info_reports()

    @xt.run_idle
    def load_crashes(self):
        self.loading = True
        self.model_crashes.clear()

        process = subprocess.run(['coredumpctl', 'list', '--no-legend', '-r', '-q'], stdout=subprocess.PIPE)
        if process.returncode != 0:
        	# in LMDE 3, coredumpctl doesn't have -r and -q options
        	process = subprocess.run(['coredumpctl', 'list', '--no-legend'], stdout=subprocess.PIPE)
        coredumps = process.stdout
        lines = coredumps.decode('utf-8').split('\n')
        for line in lines:
            # Ignore empty lines
            if not line.strip():
                continue
            line = " ".join(line.split()) # remove blank spaces between fields in the coredumpctl output
            try:
                (day, date, time, timezone, pid, uid, gid, sig, corefile, exe, *extra) = line.split(" ", 10)
            except Exception as e:
                print("coredumpctl output: '%s' could not be split: %s" % (line, e))
                continue
            # Ignore crashes for which the info isn't available
            if corefile != "present":
                continue
            timestamp = " ".join([day, date, time, timezone])
            report = CrashReport(timestamp, pid, sig, exe)
            self.model_crashes.append([day, date, time, timezone, pid, exe, report])
        self.loading = False

    def on_crash_selected(self, selection):
        if self.loading:
            return
        self.builder.get_object("crash_stack").set_visible_child_name("page0")
        self.spinner.start()
        self.spinner.show()
        self.treeview_crashes.set_sensitive(False)
        self.localfiles_button.set_sensitive(False)
        self.bugtracker_button.set_sensitive(False)
        self.pastebin_button.set_sensitive(False)
        self.buffer.set_language(self.language_manager.get_language(""))
        self.buffer.set_text("")
        os.system("rm -rf %s/*" % UNPACK_DIR)
        model, iter = selection.get_selected()
        if iter is not None:
            report = model.get_value(iter, COL_CRASH_OBJECT)
            self.dump_crash_report(report)

    @xt.run_async
    def dump_crash_report(self, report):

        os.chdir(UNPACK_DIR)
        subprocess.call(["coredumpctl", "dump", report.pid, "-o", "CoreDump"])

        # Add info about the Linux Mint release
        if os.path.exists("/etc/linuxmint/info"):
            shutil.copyfile("/etc/linuxmint/info", "LinuxMintInfo")

        # Produce a list of installed packages
        with open("Packages", "w") as f:
            subprocess.call(['dpkg', '-l'], stdout=f)

        # Produce an Inxi report
        if os.path.exists(TMP_INXI_FILE):
            os.system("cp %s Inxi" % TMP_INXI_FILE)

        executable_path = report.executable

        # Identify bug tracker
        try:
            self.bugtracker = "https://bugs.launchpad.net/"
            output = subprocess.check_output(["dpkg", "-S", executable_path]).decode("utf-8")
            if ":" in output:
                output = output.split(":")[0]
                # Check if -dbg package is missing
                dbg_name = "%s-dbg" % output
                if dbg_name in self.cache and not self.cache[dbg_name].is_installed:
                    self.buffer.set_text(_("The debug symbols are missing for %(program)s.\nPlease install %(package)s.") % {'program':output, 'package':dbg_name})
                    self.on_unpack_crash_report_finished()
                    return

                if "mate" in output or output in ["caja", "atril", "pluma", "engrampa", "eom"]:
                    self.bugtracker = "https://github.com/mate-desktop/%s/issues" % output
                elif output in self.cache:
                    pkg = self.cache[output]
                    self.bugtracker = "https://bugs.launchpad.net/%s" % output
                    for origin in pkg.installed.origins:
                        if origin.origin == "linuxmint":
                            self.bugtracker = "https://github.com/linuxmint/%s/issues" % output
                            break
        except:
            self.buffer.set_text(_("The package providing %s could not be found.\nIf you want to generate a stack trace for this crash report, please reinstall it.") % executable_path)
            self.on_unpack_crash_report_finished()
            return

        # Produce an info trace (coredumpctl info PID)
        os.system("echo '===================================================================' > StackTrace")
        os.system("echo ' Info                                                              ' >> StackTrace")
        os.system("echo '===================================================================' >> StackTrace")
        os.system("coredumpctl info %s >> StackTrace" % report.pid)

        # Produce a stack trace
        if os.path.exists("CoreDump"):
            os.system("echo '===================================================================' >> StackTrace")
            os.system("echo ' GDB Log                                                           ' >> StackTrace")
            os.system("echo '===================================================================' >> StackTrace")
            os.system("LANG=C gdb %s CoreDump --batch >> StackTrace 2>&1" % executable_path)
            os.system("echo '\n===================================================================' >> StackTrace")
            os.system("echo ' GDB Backtrace                                                     ' >> StackTrace")
            os.system("echo '===================================================================' >> StackTrace")
            os.system("LANG=C gdb %s CoreDump --batch --ex bt >> StackTrace 2>&1" % executable_path)
            os.system("echo '\n===================================================================' >> StackTrace")
            os.system("echo ' GDB Backtrace (all threads)                                       ' >> StackTrace")
            os.system("echo '===================================================================' >> StackTrace")
            os.system("LANG=C gdb %s CoreDump --batch --ex 'thread apply all bt full' --ex bt >> StackTrace 2>&1" % executable_path)

        with open("StackTrace") as f:
            text = f.read()
            if "is not at the expected address (wrong library or version mismatch" in text:
                self.buffer.set_text(_("The headers or binaries installed on your system do not match the code which was executed during the crash.\nThey probably got upgraded since.\nA stack trace is available in 'Local Files' but its content is probably inaccurate."))
            else:
                self.show_stack_info(text)

        # Archive the crash report - exclude the CoreDump as it can be very big (close to 1GB)
        os.chdir(TMP_DIR)
        subprocess.call(["tar", "caf", CRASH_ARCHIVE, "crash", "--exclude", "CoreDump"])

        self.on_unpack_crash_report_finished()

    @xt.run_idle
    def on_unpack_crash_report_finished(self):
        self.treeview_crashes.set_sensitive(True)
        self.localfiles_button.set_sensitive(True)
        self.spinner.stop()
        self.builder.get_object("crash_stack").set_visible_child_name("page1")

    @xt.run_idle
    def show_stack_info(self, text):
        self.buffer.set_language(self.language_manager.get_language("gdb-log"))
        self.buffer.set_text(text)
        self.bugtracker_button.set_sensitive(True)
        self.pastebin_button.set_sensitive(True)

    def on_button_browse_crash_report_clicked(self, button):
        os.system("xdg-open %s" % TMP_DIR)

    def on_button_open_bugtracker_clicked(self, button):
        os.system("xdg-open %s" % self.bugtracker)

    def on_button_pastebin_clicked(self, button):
        pastebin = subprocess.Popen(['/usr/bin/pastebin', os.path.join(UNPACK_DIR, "StackTrace")], stdout=subprocess.PIPE)
        output = pastebin.communicate()[0]
        output = output.split()[0] # if we have more than one URL, only use the first one
        pastebin.wait()
        subprocess.call(['xdg-open', output])

if __name__ == "__main__":
    application = MyApplication("com.linuxmint.reports", Gio.ApplicationFlags.HANDLES_COMMAND_LINE)
    application.run(sys.argv)



#!/usr/bin/python3
import gi
gi.require_version("Gtk", "3.0")
from common import clean_brand
from gi.repository import Gtk, Gdk, Pango
import pyudev
import subprocess
import xapp.threading as xt
import xapp.util

_ = xapp.util.l10n("mintreport")

COL_FULL, COL_ICON, COL_ACTIVE, COL_BUS, COL_TYPE, COL_VENDOR, COL_NAME, COL_ID, COL_DRIVER = range(9)

PCI_CLASS_MAP = {
    # 00h — Unclassified
    (0x00, 0x00): "Non-VGA unclassified device",
    (0x00, 0x01): "VGA compatible unclassified device",

    # 01h — Mass storage
    (0x01, 0x00): "SCSI storage",
    (0x01, 0x01): "IDE interface",
    (0x01, 0x02): "Floppy disk",
    (0x01, 0x03): "IPI bus",
    (0x01, 0x04): "RAID bus",
    (0x01, 0x05): "ATA",
    (0x01, 0x06): "SATA",
    (0x01, 0x07): "Serial Attached SCSI",
    (0x01, 0x08): "NVME",
    (0x01, 0x80): "Mass storage",

    # 02h — Network
    (0x02, 0x00): "Ethernet",
    (0x02, 0x01): "Token ring network",
    (0x02, 0x02): "FDDI network",
    (0x02, 0x03): "ATM network",
    (0x02, 0x04): "ISDN",
    (0x02, 0x80): "Network",

    # 03h — Display
    (0x03, 0x00): "VGA compatible",
    (0x03, 0x01): "XGA compatible",
    (0x03, 0x02): "3D",
    (0x03, 0x80): "Display",

    # 04h — Multimedia
    (0x04, 0x00): "Multimedia video",
    (0x04, 0x01): "Multimedia audio",
    (0x04, 0x02): "Computer telephony device",
    (0x04, 0x03): "Audio device",
    (0x04, 0x80): "Multimedia",

    # 05h — Memory
    (0x05, 0x00): "RAM",
    (0x05, 0x01): "Flash",
    (0x05, 0x80): "Memory",

    # 06h — Bridge device
    (0x06, 0x00): "Host bridge",
    (0x06, 0x01): "ISA bridge",
    (0x06, 0x02): "EISA bridge",
    (0x06, 0x03): "MicroChannel bridge",
    (0x06, 0x04): "PCI bridge",
    (0x06, 0x05): "PCMCIA bridge",
    (0x06, 0x06): "NuBus bridge",
    (0x06, 0x07): "CardBus bridge",
    (0x06, 0x08): "RACEway bridge",
    (0x06, 0x09): "PCI-to-PCI bridge",
    (0x06, 0x0A): "InfiniBand to PCI host bridge",
    (0x06, 0x80): "Bridge device",

    # 07h — Simple communication
    (0x07, 0x00): "Serial",
    (0x07, 0x01): "Parallel",
    (0x07, 0x02): "Multiport serial",
    (0x07, 0x03): "Modem",
    (0x07, 0x80): "Communication",

    # 08h — Base system peripheral
    (0x08, 0x00): "PIC",
    (0x08, 0x01): "DMA",
    (0x08, 0x02): "Timer",
    (0x08, 0x03): "RTC",
    (0x08, 0x04): "PCI hot-plug",
    (0x08, 0x05): "SD Host",
    (0x08, 0x06): "IOMMU",
    (0x08, 0x80): "System peripheral",

    # 09h — Input device
    (0x09, 0x00): "Keyboard",
    (0x09, 0x01): "Digitizer",
    (0x09, 0x02): "Mouse",
    (0x09, 0x03): "Scanner",
    (0x09, 0x04): "Gameport",
    (0x09, 0x80): "Input device",

    # 0Ah — Docking station
    (0x0A, 0x00): "Docking station",
    (0x0A, 0x80): "Docking station",

    # 0Bh — Processor
    (0x0B, 0x00): "386",
    (0x0B, 0x01): "486",
    (0x0B, 0x02): "Pentium",
    (0x0B, 0x03): "Pentium Pro",
    (0x0B, 0x10): "Alpha",
    (0x0B, 0x20): "PowerPC",
    (0x0B, 0x30): "MIPS",
    (0x0B, 0x40): "Co-processor",
    (0x0B, 0x80): "Processor",

    # 0Ch — Serial bus
    (0x0C, 0x00): "FireWire (IEEE 1394)",
    (0x0C, 0x01): "ACCESS bus",
    (0x0C, 0x02): "SSA",
    (0x0C, 0x03): "USB",
    (0x0C, 0x04): "Fibre Channel",
    (0x0C, 0x05): "SMBus",
    (0x0C, 0x06): "InfiniBand",
    (0x0C, 0x07): "IPMI",
    (0x0C, 0x08): "SERCOS",
    (0x0C, 0x09): "CANbus",
    (0x0C, 0x80): "Serial bus",

    # 0Dh — Wireless
    (0x0D, 0x00): "iRDA",
    (0x0D, 0x01): "Consumer IR",
    (0x0D, 0x10): "RF",
    (0x0D, 0x11): "Bluetooth",
    (0x0D, 0x12): "Broadband",
    (0x0D, 0x20): "Ethernet (802.11a/b/g)",
    (0x0D, 0x80): "Wireless",

    # 0Eh — Intelligent I/O
    (0x0E, 0x00): "I2O",

    # 0Fh — Satellite communication
    (0x0F, 0x01): "Satellite TV",
    (0x0F, 0x02): "Satellite audio",
    (0x0F, 0x03): "Satellite voice",
    (0x0F, 0x04): "Satellite data",

    # 10h — Encryption / decryption
    (0x10, 0x00): "Network and computing encr/decr",
    (0x10, 0x10): "Entertainment encr/decr",
    (0x10, 0x80): "Encryption",

    # 11h — Data acquisition / signal processing
    (0x11, 0x00): "DPIO module",
    (0x11, 0x01): "Performance counters",
    (0x11, 0x10): "Communication synchronizer",
    (0x11, 0x20): "Signal processing management",
    (0x11, 0x80): "Data acquisition",
}

def get_icon_for_device(info):
    """Return an XApp symbolic icon name for a PCI class string."""
    dev_class = info['class'].lower()
    dev_name = info['device'].lower()

    if "vga" in dev_class or "display" in dev_class or "gpu" in dev_class:
        return "xsi-display-symbolic"
    if "network" in dev_class and ("wifi" in dev_name or "wireless" in dev_name):
        return "xsi-network-wireless-symbolic"
    if "network" in dev_class or "ethernet" in dev_class:
        return "xsi-network-wired-symbolic"
    if "nvme" in dev_class or " nvme " in dev_name:
        return "xsi-drive-harddisk-solidstate-symbolic"
    if "storage" in dev_class or "sata" in dev_class or "ahci" in dev_class:
        return "xsi-drive-harddisk-symbolic"
    if "audio" in dev_class or "sound" in dev_class:
        return "xsi-audio-card-symbolic"
    if "serial" in dev_class and " usb " in dev_name:
        return "xsi-usb-symbolic"
    if "multimedia" in dev_class:
        return "xsi-audio-card-symbolic"
    if "thermal" in dev_name:
        return "xsi-temperature-symbolic"
    return "xsi-cpu-symbolic"


def get_pci_devices():
    """Collect PCI device information using pyudev."""
    context = pyudev.Context()
    devices = []

    def decode(v): return v.decode().strip() if hasattr(v, "decode") else v

    for dev in context.list_devices(subsystem="pci"):
        try:
            vendor_id = dev.attributes.asstring("vendor")[2:]
            device_id = dev.attributes.asstring("device")[2:]
        except Exception:
            vendor_id = device_id = None

        active = False
        power = dev.attributes.get("power/runtime_status")
        power = decode(power)
        if power == "active":
            active = True
        revision = dev.attributes.get("revision")
        revision = decode(revision)
        if revision.startswith("0x"):
            revision = revision[2:].lower()
        else:
            revision = f"{int(revision):02x}"
        if revision == "00":
            revision = ""
        else:
            revision = f"(rev {revision})"

        raw_class = dev.attributes.get("class")
        class_name = None

        if raw_class:
            val = int(raw_class.decode().strip(), 16)
            base, sub = (val >> 16) & 0xFF, (val >> 8) & 0xFF
            class_name = PCI_CLASS_MAP.get((base, sub))

        info = {
            "address": dev.sys_name,
            "vendor": dev.get("ID_VENDOR_FROM_DATABASE") or "",
            "device": dev.get("ID_MODEL_FROM_DATABASE") or f"Device {device_id}",
            "class": class_name or dev.get("ID_PCI_CLASS_FROM_DATABASE") or "",
            "vendor_id": vendor_id,
            "device_id": device_id,
            "driver": dev.driver or "",
            "active": active,
            "revision": revision
        }

        info["icon"] = get_icon_for_device(info)
        info["id_string"] = f"{vendor_id}:{device_id}" if vendor_id and device_id else ""

        devices.append(info)

    return devices

class PCIListWidget(Gtk.Box):
    def __init__(self):
        super().__init__(orientation=Gtk.Orientation.VERTICAL, spacing=6)
        self.set_border_width(6)

        self.store = Gtk.ListStore(str, str, bool, str, str, str, str, str, str, str)

        self.treeview = Gtk.TreeView(model=self.store)
        self.treeview.set_headers_visible(True)
        self.treeview.set_enable_tree_lines(False)
        self.treeview.set_grid_lines(Gtk.TreeViewGridLines.NONE)
        self.treeview.set_property("expand", True)
        self.treeview.set_headers_clickable(True)

        # Bus
        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn(_("Bus"), renderer, text=COL_BUS)
        column.set_sort_column_id(COL_BUS)
        column.set_resizable(True)
        column.add_attribute(renderer, "sensitive", COL_ACTIVE)
        self.treeview.append_column(column)

        # Type
        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn(_("Type"), renderer, text=COL_TYPE)
        column.set_sort_column_id(COL_TYPE)
        column.set_resizable(True)
        column.add_attribute(renderer, "sensitive", COL_ACTIVE)
        self.treeview.append_column(column)

        # Vendor
        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn(_("Brand"), renderer, text=COL_VENDOR)
        column.set_sort_column_id(COL_VENDOR)
        column.set_resizable(True)
        column.add_attribute(renderer, "sensitive", COL_ACTIVE)
        self.treeview.append_column(column)

        # Icon + Name
        column = Gtk.TreeViewColumn(_("Device"))
        column.set_sort_column_id(COL_NAME)
        icon_renderer = Gtk.CellRendererPixbuf()
        renderer = Gtk.CellRendererText()
        renderer.set_property("ellipsize", 3)
        renderer.set_property("ypad", 6)
        icon_renderer.set_property("xpad", 2)
        column.pack_start(icon_renderer, False)
        column.pack_start(renderer, True)
        column.add_attribute(icon_renderer, "icon-name", COL_ICON)
        column.add_attribute(renderer, "text", COL_NAME)
        renderer.set_property("ellipsize", Pango.EllipsizeMode.END)
        column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
        self.treeview.append_column(column)
        column.set_expand(True)
        column.set_resizable(True)
        column.add_attribute(renderer, "sensitive", COL_ACTIVE)

        # ID
        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn(_("ID"), renderer, text=COL_ID)
        column.set_sort_column_id(COL_ID)
        column.set_resizable(True)
        column.add_attribute(renderer, "sensitive", COL_ACTIVE)
        self.treeview.append_column(column)

        # Driver
        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn(_("Driver"), renderer, text=COL_DRIVER)
        column.set_sort_column_id(COL_DRIVER)
        column.set_resizable(True)
        column.add_attribute(renderer, "sensitive", COL_ACTIVE)
        self.treeview.append_column(column)

        self.store.set_sort_column_id(COL_BUS, Gtk.SortType.ASCENDING)

        scrolled = Gtk.ScrolledWindow()
        scrolled.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
        scrolled.set_shadow_type(Gtk.ShadowType.IN)
        scrolled.add(self.treeview)
        self.pack_start(scrolled, True, True, 0)

        button_box = Gtk.Box(spacing=6)
        self.copy_id_btn = Gtk.Button(label=_("Copy the ID"))
        self.copy_row_btn = Gtk.Button(label=_("Copy the entire row"))
        self.copy_id_btn.set_sensitive(False)
        self.copy_row_btn.set_sensitive(False)
        button_box.pack_end(self.copy_id_btn, False, False, 0)
        button_box.pack_end(self.copy_row_btn, False, False, 0)
        self.pack_start(button_box, False, False, 0)

        selection = self.treeview.get_selection()
        selection.connect("changed", self.on_selection_changed)
        self.copy_id_btn.connect("clicked", self.on_copy_id_clicked)
        self.copy_row_btn.connect("clicked", self.on_copy_row_clicked)

    def load(self):
        self.populate_pci_devices()
        self.setup_monitor()

    def on_selection_changed(self, selection):
        model, treeiter = selection.get_selected()
        active = bool(treeiter)
        self.copy_id_btn.set_sensitive(active)
        self.copy_row_btn.set_sensitive(active)

    def on_copy_id_clicked(self, button):
        selection = self.treeview.get_selection()
        model, treeiter = selection.get_selected()
        if not treeiter:
            return
        device_id = model[treeiter][COL_ID]
        clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
        clipboard.set_text(device_id, -1)
        subprocess.Popen(['notify-send', '-i', 'xsi-dialog-information-symbolic', _("Device ID copied"), _("The device ID was copied into your clipboard.")])

    def on_copy_row_clicked(self, button):
        selection = self.treeview.get_selection()
        model, treeiter = selection.get_selected()
        if not treeiter:
            return
        text = f"{model[treeiter][COL_FULL]}"
        clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
        clipboard.set_text(text, -1)
        subprocess.Popen(['notify-send', '-i', 'xsi-dialog-information-symbolic', _("Device information copied"), _("The device information was copied into your clipboard.")])

    @xt.run_async
    def populate_pci_devices(self):
        """Enumerate PCI devices in a background thread."""
        devices = get_pci_devices()
        self._update_pci_list(devices)

    @xt.run_idle
    def _update_pci_list(self, devices):
        """Update TreeStore safely inside GTK main loop."""
        self.store.clear()

        for d in devices:
            name = f"{d['device']} {d['revision']}"

            vendor = clean_brand(d['vendor'])
            full_info = f"{d['address']} {d['class']} {d['vendor']} {d['device']} {d['revision']} {d['driver']}"
            iter_ = self.store.append()
            self.store.set_value(iter_, COL_FULL, full_info)
            self.store.set_value(iter_, COL_ICON, d["icon"])
            self.store.set_value(iter_, COL_NAME, name)
            self.store.set_value(iter_, COL_BUS, d['address'])
            self.store.set_value(iter_, COL_TYPE, d['class'])
            self.store.set_value(iter_, COL_VENDOR, vendor)
            self.store.set_value(iter_, COL_ACTIVE, d["active"])
            self.store.set_value(iter_, COL_ID, d["id_string"])
            self.store.set_value(iter_, COL_DRIVER, d["driver"])

    def setup_monitor(self):
        """Watch for live PCI changes via udev monitor."""
        context = pyudev.Context()
        monitor = pyudev.Monitor.from_netlink(context)
        monitor.filter_by(subsystem="pci")

        def on_event(action, device):
            # Just trigger background re-enumeration
            self.populate_pci_devices()

        self.observer = pyudev.MonitorObserver(monitor, callback=on_event, name="pci-monitor")
        self.observer.start()


# -------------------------------------------------------------------
# Standalone test window
# -------------------------------------------------------------------

if __name__ == "__main__":
    win = Gtk.Window(title="PCI Devices")
    viewer = PCIListWidget()
    viewer.load()
    win.add(viewer)
    win.set_default_size(800, 400)
    win.connect("destroy", Gtk.main_quit)
    win.show_all()
    Gtk.main()



import gettext
import gi
import os
import subprocess
import re
import json

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk

from mintreport import InfoReport, InfoReportAction

class Report(InfoReport):

    def __init__(self):

        gettext.install("mintreport", "/usr/share/locale", names="ngettext")

        self.title = _("Install hardware drivers")
        self.icon = "mintreport-hardware-device-symbolic"
        self.has_ignore_button = True

    def get_cpu_name(self):
        with open("/proc/cpuinfo") as cpuinfo:
            for line in cpuinfo:
                if "model name" in line:
                    return re.sub( ".*model name.*:", "", line, 1).strip()
        return _("Processor")

    def construct_name(self, device):
        model_name = device.get('model', None)
        vendor_name = device.get('vendor', None)
        device_name = None

        is_cpu = False
        if "intel-microcode" in device['drivers'] or "amd64-microcode" in device['drivers']:
            is_cpu = True
        if is_cpu:
            device_name = self.get_cpu_name()
        elif vendor_name is None and model_name is None:
            device_name = _("Unknown")
        elif vendor_name is None:
            device_name = model_name
        elif model_name is None:
            device_name = _("%s (unknown)" % vendor_name)
        else:
            device_name = "%s: %s" % (vendor_name, model_name)
        return device_name

    def is_pertinent(self):
        # Defines whether this report should show up
        self.drivers = []

        # ubuntu-drivers and UbuntuDrivers module share the same package; if it's
        # missing, we're on lmde.
        if not os.path.exists("/usr/bin/ubuntu-drivers"):
            return False

        try:
            output = subprocess.check_output(
                ["python3",
                 "-c",
                 "import json; from UbuntuDrivers import detect; detect.apt_pkg.init(); d = json.dumps(detect.system_device_drivers()); print(d)"],
                stderr=subprocess.DEVNULL).decode("UTF-8")

            devices = json.loads(output)
            for device_id in devices:
                device = devices[device_id]
                device_name = self.construct_name(device)
                if "virtualbox" in device_name.lower() or "vmware" in device_name.lower():
                    print("Ignoring %s" % device_name)
                    # Ignore them, they're not recommended
                    continue
                if 'manual_install' in device.keys() and device['manual_install'] == True:
                    print("Ignoring device: %s (manual_install)" % device_name)
                    continue
                packages = []
                for driver_name in device['drivers']:
                    driver = device['drivers'][driver_name]
                    if 'builtin' in driver.keys() and driver['builtin'] == True:
                        print("Ignoring driver: %s (builtin)" % driver)
                        continue
                    packages.append(driver_name)
                any_installed = False
                for pkg in packages:
                    # Check if the package is installed
                    if os.path.exists("/var/lib/dpkg/info/%s.list" % pkg):
                        any_installed = True
                        break
                if not any_installed:
                    self.drivers.append(device_name)
            return (len(self.drivers) > 0)
        except Exception as e:
            print ("Failed to assess drivers availability: %s" % e)
            return False

    def get_descriptions(self):
        # Return the descriptions
        descriptions = []
        descriptions.append(_("Hardware drivers are available for the following devices:"))
        descriptions.append("\n".join(self.drivers))
        return descriptions

    def get_actions(self):
        # Return available actions
        actions = []
        action = InfoReportAction(label=_("Launch the Driver Manager"), callback=self.callback)
        action.set_style(Gtk.STYLE_CLASS_SUGGESTED_ACTION)
        actions.append(action)
        return actions

    def callback(self, data):
        subprocess.run(["driver-manager"])
        # don't reload
        return False

if __name__ == "__main__":
    report = Report()
    print(report.is_pertinent())



#!/usr/bin/python3
import glob
import importlib
import os
import sys
from pathlib import Path

DATA_DIR = "/usr/share/linuxmint/mintreport"
INFO_DIR = os.path.join(DATA_DIR, "reports")
TMP_DIR = "/tmp/mintreport"

class InfoReportContainer():
    def __init__(self, uuid, path):
        self.uuid = uuid
        self.path = path
        sys.path.insert(0, path)
        import MintReportInfo
        importlib.reload(MintReportInfo)
        self.instance = MintReportInfo.Report()
        sys.path.remove(path)

def prefix_version(version):
    if version == "":
        return ""
    elif version[0].isnumeric():
        return f"v{version}"
    else:
        return version

def clean_brand(brand):
    return BRAND_MAP.get(brand, brand)

def read_efi(name):
    """Read a UEFI variable (returns None if missing or unreadable)."""
    paths = glob.glob(f"/sys/firmware/efi/efivars/{name}-*")
    if not paths:
        return None
    try:
        with open(paths[0], "rb") as f:
            data = f.read()
        # First 4 bytes are attributes; next byte is the actual value
        return data[4]
    except Exception:
        return None

def read_dmi(field):
    path = Path(f"/sys/class/dmi/id/{field}")
    if not path.exists():
        return ""
    try:
        value = path.read_text().strip()
        if value.lower() in BAD_DMI_VALUES:
            value = ""
        return value
    except Exception as e:
        print("Could not read", path)
        return ""

BAD_DMI_VALUES = [
    "default string",
    "to be filled by o.e.m.",
    "not specified",
    "none",
    "n/a",
    "unknown",
]

BRAND_MAP = {
    "Adaptec": "Adaptec",
    "Advanced Micro Devices, Inc. [AMD/ATI]": "AMD",
    "Advanced Micro Devices, Inc.": "AMD",
    "Altera Corporation": "Altera",
    "American Megatrends Inc.": "AMI",
    "Apple Inc.": "Apple",
    "ASMedia Technology Inc.": "ASMedia",
    "ASUSTeK COMPUTER INC.": "ASUS",
    "AverMedia Technologies, Inc.": "AverMedia",
    "Blackmagic Design": "Blackmagic",
    "Broadcom / Avago": "Broadcom",
    "Broadcom / LSI": "LSI",
    "Broadcom Inc. and subsidiaries": "Broadcom",
    "Broadcom Limited": "Broadcom",
    "C-Media Electronics Inc": "C-Media",
    "Creative Labs": "Creative",
    "Creative Technology Ltd.": "Creative",
    "Crucial Technology": "Crucial",
    "Compulab Ltd.": "Compulab",
    "Dell Inc.": "Dell",
    "Dynabook Inc.": "Dynabook",
    "Elgato Systems": "Elgato",
    "Freescale Semiconductor Inc": "Freescale",
    "Fujitsu / Fujitsu Siemens Computers": "Fujitsu",
    "Gigabyte Technology Co., Ltd.": "Gigabyte",
    "GoPro, Inc.": "GoPro",
    "Hewlett-Packard / HP": "HP",
    "Huawei Technologies Co., Ltd.": "Huawei",
    "HUAWEI": "Huawei",
    "innotek GmBH": "innotek",
    "Intel Corporation (Network)": "Intel",
    "Intel Corporation": "Intel",
    "Kingston Technology Company": "Kingston",
    "Lattice Semiconductor Corp.": "Lattice",
    "Marvell Technology Group Ltd.": "Marvell",
    "MediaTek Inc.": "MediaTek",
    "Mellanox Technologies": "Mellanox",
    "Microchip Technology Inc.": "Microchip",
    "Micron Technology": "Micron",
    "Micro-Star International Co., Ltd.": "MSI",
    "Microsoft Corporation": "Microsoft",
    "NEC Corporation": "NEC",
    "NVIDIA Corporation": "NVIDIA",
    "NXP Semiconductors": "NXP",
    "Oracle Corporation": "Oracle",
    "Panasonic Corporation": "Panasonic",
    "Phison Electronics Corporation": "Phison",
    "Qualcomm Atheros": "Atheros",
    "Qualcomm Technologies, Inc": "Qualcomm",
    "Realtek Semiconductor Co., Ltd.": "Realtek",
    "Realtek Semiconductor Corp.": "Realtek",
    "Renesas Technology Corp.": "Renesas",
    "Samsung Electronics Co Ltd": "Samsung",
    "Samsung Electronics Co., Ltd.": "Samsung",
    "SanDisk Corp.": "SanDisk",
    "Sony Corporation": "Sony",
    "Texas Instruments": "TI",
    "VIA Technologies, Inc.": "VIA",
    "VMware, Inc.": "VMware",
    "Western Digital Corporation": "WD",
    "Western Digital": "WD",
    "Xilinx Corporation": "Xilinx",
}



import gettext
import os
from pathlib import Path
import webbrowser

from mintreport import InfoReport, InfoReportAction

class Report(InfoReport):

    def __init__(self):

        gettext.install("mintreport", "/usr/share/locale", names="ngettext")

        self.title = _("Perform usrmerge conversion")
        self.icon = "xsi-folder-symbolic"
        self.has_ignore_button = True

    def is_pertinent(self):
        # from convert-usrmerge script
        directories = ["bin", "sbin", "lib", "libx32", "lib64", "lib32"]

        for dirname in directories:
            link_path = Path(os.path.join("/", dirname))
            target_path = Path(os.path.join("/", "usr", dirname))

            if (not link_path.exists()) and (not target_path.exists()):
                # print("skipping: %s (doesn't exist)" % link_path)
                continue

            # print("checking: %s --> %s" % (link_path, target_path))

            if not (link_path.is_symlink() and link_path.samefile(target_path)):
                return True
        return False

    def get_descriptions(self):
        # Return the descriptions
        descriptions = []
        descriptions.append(_("We recommend you convert your system with usrmerge."))
        descriptions.append(_("This is done already for new installations of Linux Mint."))
        descriptions.append(_("To convert your system, open a terminal and type:"))
        descriptions.append("\n<span font_family='monospace'>apt install usrmerge</span>\n")
        descriptions.append(_("You should reboot the computer once this completes."))
        return descriptions

    def get_actions(self):
        # Return available actions
        actions = []
        action = InfoReportAction(label=_("More information"), callback=self.more_info)
        actions.append(action)
        return actions

    def more_info(self, data=None):
        webbrowser.open("https://www.freedesktop.org/wiki/Software/systemd/TheCaseForTheUsrMerge")

if __name__ == "__main__":
    report = Report()
    print(report.is_pertinent())



#!/usr/bin/python3
import gi
import os
import pyudev
import re
import shutil
import subprocess
import xapp.SettingsWidgets as Xs
import xapp.threading as xt
import xapp.util
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk
from common import read_dmi, read_efi, clean_brand
from pathlib import Path

_ = xapp.util.l10n("mintreport")


def run_cmd(cmd):
    try:
        return subprocess.check_output(cmd, text=True, stderr=subprocess.DEVNULL)
    except Exception:
        return ""

# --- OpenGL detection ---
def detect_opengl():
    """
    Returns True if OpenGL is hardware-accelerated (not llvmpipe/swrast/softpipe).
    """
    out = run_cmd(["glxinfo", "-B"])
    if not out:
        return False

    # Check for direct rendering
    direct = bool(re.search(r"direct rendering:\s*Yes", out, re.IGNORECASE))

    # Extract renderer string
    m = re.search(r"OpenGL renderer string:\s*(.*)", out)
    renderer = m.group(1).strip() if m else ""

    # Detect software renderers
    software = any(x in renderer.lower() for x in ("llvmpipe", "softpipe", "swrast", "mesa offscreen"))

    return direct and not software

def detect_vulkan_acceleration():
    """Return True if any Vulkan GPU is hardware-accelerated."""
    try:
        out = subprocess.check_output(
            ["vulkaninfo", "--summary"],
            text=True, stderr=subprocess.DEVNULL, timeout=3
        )
    except Exception:
        return None  # vulkaninfo not available or failed

    # Split per GPU block
    gpus = re.split(r"GPU\d+:", out)
    accel_found = False

    for gpu in gpus:
        if not gpu.strip():
            continue
        # Ignore software drivers like llvmpipe
        if re.search(r"lavapipe|llvmpipe", gpu, re.IGNORECASE):
            continue
        # If we find a valid hardware vendor/device ID
        if re.search(r"vendorID\s*=\s*0x[0-9a-f]+", gpu, re.IGNORECASE):
            accel_found = True
            break

    return accel_found

def detect_gles():
    result = False
    out = run_cmd(["es2_info"]) # part of mesa-utils
    if out:
        out = out.lower()
        if "llvmpipe" not in out and "softpipe" not in out:
            result = True
    return result

def get_default_gpu_id():
    drm_path = Path("/sys/class/drm")
    # Only match card0, card1, etc. - not connector entries like card0-DP-1, card1-HDMI-A-1
    cards = sorted(c for c in drm_path.iterdir() if re.match(r'^card\d+$', c.name))
    for card_dir in cards:
        dev_dir = card_dir / "device"
        vendor_file = dev_dir / "vendor"
        device_file = dev_dir / "device"
        if not (vendor_file.exists() and device_file.exists()):
            continue

        # Detect display connection (for default GPU)
        has_display = any(
            (card_dir / conn / "status").exists() and
            (card_dir / conn / "status").read_text().strip() == "connected"
            for conn in card_dir.iterdir() if conn.is_dir()
        )

        vendor = vendor_file.read_text().strip()
        device = device_file.read_text().strip()
        pci_id = f"{vendor}:{device}".replace("0x", "")

        if has_display:
            return pci_id

    # Fallback: return the first card's PCI ID
    if cards:
        dev_dir = cards[0] / "device"
        vendor_file = dev_dir / "vendor"
        device_file = dev_dir / "device"
        if vendor_file.exists() and device_file.exists():
            vendor = vendor_file.read_text().strip()
            device = device_file.read_text().strip()
            return f"{vendor}:{device}".replace("0x", "")

    return None

def get_pci_device(default_id):
    context = pyudev.Context()
    devices = []

    def decode(v): return v.decode().strip() if hasattr(v, "decode") else v

    for dev in context.list_devices(subsystem="pci"):
        try:
            vendor = dev.attributes.asstring("vendor")[2:]
            device = dev.attributes.asstring("device")[2:]
            pci_id = f"{vendor}:{device}"
            if pci_id != default_id:
                continue
        except Exception as e:
            print(e)

        return dev

def detect_video_acceleration():
    result = []
    # VA-API
    try:
        out = subprocess.check_output(
            ["vainfo"],
            text=True,
            stderr=subprocess.STDOUT,
            timeout=3,
        )
        if "Driver version" in out or re.search(r"VAProfile", out):
            result.append("VA-API")
    except Exception as e:
        pass
        #print(e)

    # VDPAU
    try:
        out = subprocess.check_output(
            ["vdpauinfo"],
            text=True,
            stderr=subprocess.STDOUT,
            timeout=3,
        )
        if re.search(r"VDPAU Driver", out) and not re.search(r"llvmpipe", out, re.IGNORECASE):
            result.append("VDPAU")
    except Exception as e:
        pass
        #print(e)

    return result

def get_gpu_driver_version(driver):
    path = Path(f"/sys/module/{driver}/version")
    if path.exists():
        return path.read_text().strip()  # e.g. NVIDIA
    if driver in ("i915", "amdgpu", "nouveau"):
        # Built-in drivers → no version file
        return None
    # fallback: modinfo
    try:
        out = subprocess.check_output(["modinfo", driver], text=True)
        m = re.search(r"version:\s*([^\n]+)", out)
        if m:
            return m.group(1).strip()
    except Exception:
        pass
    return None

def bool_to_accel(boolean):
    if boolean:
        return _("Enabled")
    else:
        return _("Disabled (Software rendering)")

class GPUListWidget(Xs.SettingsPage):
    def __init__(self):
        super().__init__()
        self.set_spacing(24)
        self.set_margin_left(0)
        self.set_margin_right(0)
        self.set_margin_top(0)
        self.set_margin_bottom(0)
        self.section_gpu = self.add_section(_("Default GPU"))
        self.section_acceleration = self.add_section(_("Hardware Acceleration"))

    @xt.run_async
    def load(self):
        gl = detect_opengl()
        es = detect_gles()
        vk = detect_vulkan_acceleration()
        pci_id = get_default_gpu_id()
        device = get_pci_device(pci_id)
        vendor = device.get("ID_VENDOR_FROM_DATABASE") or ""
        name = device.get("ID_MODEL_FROM_DATABASE") or ""
        driver = device.driver
        driver_version = get_gpu_driver_version(driver)
        video = detect_video_acceleration()
        if len(video) == 0:
            video = _("Disabled (Software rendering)")
        else:
            video = _("Enabled (%s)") % ", ".join(video)
        info_gpu = []
        info_gpu.append([_('Brand'), clean_brand(vendor)])
        info_gpu.append([_('Name'), name])
        info_gpu.append([_('PCI ID'), pci_id])
        info_gpu.append([_('Driver'), driver])
        if driver_version:
            info_gpu.append([_('Driver Version'), driver_version])
        info_acceleration = []
        info_acceleration.append(['OpenGL', bool_to_accel(gl)])
        info_acceleration.append(['OpenGL ES', bool_to_accel(es)])
        info_acceleration.append(['Vulkan', bool_to_accel(vk)])
        info_acceleration.append([_('Video Playback'), video])
        self.update_ui(info_gpu, self.section_gpu)
        self.update_ui(info_acceleration, self.section_acceleration)

    @xt.run_idle
    def update_ui(self, info, section):
        for (key, value) in info:
            widget = Xs.SettingsWidget()
            widget.set_spacing(40)
            labelKey = Gtk.Label.new(key)
            widget.pack_start(labelKey, False, False, 0)
            labelKey.get_style_context().add_class("dim-label")
            labelValue = Gtk.Label.new(value)
            labelValue.set_selectable(True)
            labelValue.set_line_wrap(True)
            widget.pack_end(labelValue, False, False, 0)
            section.add_row(widget)
        self.show_all()

# -------------------------------------------------------------------
# Standalone test window
# -------------------------------------------------------------------

if __name__ == "__main__":
    win = Gtk.Window()
    viewer = GPUListWidget()
    viewer.load()
    win.add(viewer)
    win.set_default_size(800, 400)
    win.connect("destroy", Gtk.main_quit)
    win.show_all()
    Gtk.main()



import configobj
import gettext
import time

from datetime import datetime
from mintreport import InfoReport, InfoReportAction

class Report(InfoReport):

    def __init__(self):

        gettext.install("mintreport", "/usr/share/locale", names="ngettext")

        self.icon = "xsi-software-update-urgent-symbolic"
        self.has_ignore_button = False

    def is_pertinent(self):
        # Defines whether this report should show up
        self.eol_date = None
        base_codename = None
        base_distro = None

        config = configobj.ConfigObj("/etc/os-release")
        for base_distribution in ['ubuntu', 'debian']:
            if '%s_CODENAME' % base_distribution.upper() in config:
                base_codename = config.get('%s_CODENAME' % base_distribution.upper())
                base_distro = base_distribution
                break

        distro_info = open("/usr/share/distro-info/%s.csv" % base_distro, "r").readlines()
        for line in distro_info[1:]:
            elements = line.split(",")
            if len(elements) >= 6:
            	codename = elements[2]
            	eol = elements[5]
            	if codename != base_codename:
                	continue

            	self.eol_date = time.mktime(time.strptime(eol.rstrip(), '%Y-%m-%d'))
            	self.eol_date = datetime.fromtimestamp(self.eol_date)

        if not self.eol_date:
        	print ("Could not find the EOL date for %s" % base_codename)
        	return False

        self.days_before_eol = (self.eol_date - datetime.now()).days
        if self.days_before_eol < 90:
            if self.days_before_eol > 0:
                self.title = _("Your version of Linux Mint will soon reach End-Of-Life (EOL)")
                self.description = gettext.ngettext('It will stop receiving support and updates in a day.', \
                                            'It will stop receiving support and updates in {num_days} days.', self.days_before_eol) \
                                            .format(num_days=self.days_before_eol)
            else:
                self.title = _("Your version of Linux Mint is no longer supported")
                self.description = gettext.ngettext('It reached End-Of-Life (EOL) yesterday.', \
                                            'It reached End-Of-Life (EOL) {num_days} days ago.', self.days_before_eol) \
                                            .format(num_days=(self.days_before_eol * -1))
            return True
        else:
            return False

    def get_descriptions(self):
        # Return the descriptions
        descriptions = []
        descriptions.append(self.description)
        descriptions.append(_("Visit %s to find information about newer releases.") % "<a href='https://www.linuxmint.com'>https://www.linuxmint.com</a>")
        return descriptions

    def get_actions(self):
        # Return available actions
        actions = []
        return actions

if __name__ == "__main__":
    report = Report()
    print(report.is_pertinent())



#!/usr/bin/python3
import gi
import os
import xapp.SettingsWidgets as Xs
import xapp.threading as xt
import xapp.util
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk
from common import read_dmi, read_efi, clean_brand

_ = xapp.util.l10n("mintreport")

class BIOSListWidget(Xs.SettingsPage):
    def __init__(self):
        super().__init__()
        self.set_spacing(24)
        self.set_margin_left(0)
        self.set_margin_right(0)
        self.set_margin_top(0)
        self.set_margin_bottom(0)
        self.section_bios = self.add_section(_("BIOS"))
        self.section_motherboard = self.add_section(_("Motherboard"))

    @xt.run_async
    def load(self):
        # Don't load chassis info. It's rarely useful
        # 1. often the same as product info
        # 2. often missing
        # 3. oten just describes what the user already knows (laptop, desktop..)

        infos_bios = []
        infos_bios.append([_('Brand'), clean_brand(read_dmi('bios_vendor'))])
        infos_bios.append([_('Version'), read_dmi('bios_version')])
        infos_bios.append([_('Release'), read_dmi('bios_release')])
        infos_bios.append([_('Release Date'), read_dmi('bios_date')])

        if not os.path.exists("/sys/firmware/efi"):
            infos_bios.append([_('Boot Mode'), 'BIOS'])
            infos_bios.append([_('Secure Boot'), _("Disabled")])
        else:
            infos_bios.append([_('Boot Mode'), 'UEFI'])
            if read_efi("SecureBoot") == 1:
                infos_bios.append([_('Secure Boot'), _("Enabled")])
                setup_mode = read_efi("SetupMode")
                if setup_mode == 1:
                    infos_bios.append([_('Secure Boot mode'), _("Setup Mode (keys not enrolled)")])
                elif setup_mode == 0:
                    infos_bios.append([_('Secure Boot mode'), _("User Mode (normal)")])
            else:
                infos_bios.append([_('Secure Boot'), _("Disabled")])

        infos_motherboard = []
        infos_motherboard.append([_('Brand'), clean_brand(read_dmi('board_vendor'))])
        infos_motherboard.append([_('Name'), read_dmi('board_name')])
        infos_motherboard.append([_('Version'), read_dmi('board_version')])

        self.update_ui(infos_bios, infos_motherboard)

    @xt.run_idle
    def update_ui(self, infos_bios, infos_motherboard):
        for (key, value) in infos_bios:
            widget = Xs.SettingsWidget()
            widget.set_spacing(40)
            labelKey = Gtk.Label.new(key)
            widget.pack_start(labelKey, False, False, 0)
            labelKey.get_style_context().add_class("dim-label")
            labelValue = Gtk.Label.new(value)
            labelValue.set_selectable(True)
            labelValue.set_line_wrap(True)
            widget.pack_end(labelValue, False, False, 0)
            self.section_bios.add_row(widget)
        for (key, value) in infos_motherboard:
            widget = Xs.SettingsWidget()
            widget.set_spacing(40)
            labelKey = Gtk.Label.new(key)
            widget.pack_start(labelKey, False, False, 0)
            labelKey.get_style_context().add_class("dim-label")
            labelValue = Gtk.Label.new(value)
            labelValue.set_selectable(True)
            labelValue.set_line_wrap(True)
            widget.pack_end(labelValue, False, False, 0)
            self.section_motherboard.add_row(widget)
        self.show_all()

# -------------------------------------------------------------------
# Standalone test window
# -------------------------------------------------------------------

if __name__ == "__main__":
    win = Gtk.Window(title="BIOS")
    viewer = BIOSListWidget()
    viewer.load()
    win.add(viewer)
    win.set_default_size(800, 400)
    win.connect("destroy", Gtk.main_quit)
    win.show_all()
    Gtk.main()



#!/usr/bin/python3
import gi
import pyudev
import subprocess
import threading
import xapp.threading as xt
import xapp.util
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gdk, Pango

_ = xapp.util.l10n("mintreport")

COL_NAME, COL_ID, COL_SPEED, COL_POWER, COL_SENSITIVE, COL_IS_CONTROLLER, COL_MA_VAL, COL_ICON_NAME, COL_VERSION, COL_POWER_ICON, COL_IS_BUS_POWERED, COL_IS_HUB = range(12)

def decode_power(device):
    mA_val = 0
    attr = "80"
    attr_val = 0x00
    try:
        with open(f"{device.sys_path}/bMaxPower") as f:
            mA_val = int(f.read().strip().lower().replace("ma", ""))
    except FileNotFoundError:
        pass
    try:
        with open(f"{device.sys_path}/bmAttributes") as f:
            attr = f.read().strip()
            attr_val = int(attr, 16) if attr.startswith("0x40") else int(attr)
    except Exception:
        attr_val = 0x80
    is_bus_powered = False if (attr_val & 0x40) else True
    return is_bus_powered, mA_val

def speed_label(speed_mbps):
    try:
        val = int(speed_mbps)
    except Exception:
        return speed_mbps
    if val >= 10000:
        return _("%d Gbps") % val
    else:
        return _("%d Mbps") % val

def usb_limit(version):
    try:
        ver = float(version)
    except Exception:
        ver = 2.0
    return 900 if ver >= 3.0 else 500

def get_device_icon(device, vid_str):
    icon_name = "xsi-drive-harddisk-usb-symbolic"

    if vid_str == "1d6b": # Controller
        return icon_name

    try:
        device_class = device.attributes.get("bDeviceClass").decode().lower()
        device_subclass = device.attributes.get("bDeviceSubClass").decode()
        device_protocol = device.attributes.get("bDeviceProtocol").decode()
        match (device_class, device_subclass):
            case ("09", _): # Hub
                icon_name = "xsi-drive-harddisk-usb-symbolic"
            case ("08", _): # Storage
                icon_name = "xsi-usb-stick-symbolic"
            case ("03", "01"):
                if device_protocol == "01": # Keyboard
                    icon_name = "xsi-input-keyboard-symbolic"
                elif device_protocol == "02": # Mouse
                    icon_name = "xsi-input-mouse-symbolic"
            case ("0e", _): # Camera
                icon_name = "xsi-camera-video-symbolic"
            case ("01", _): # Audio Interface
                icon_name = "xsi-audio-card-symbolic"
            case ("02", "06"): # Ethernet Adapter
                icon_name = "xsi-network-wired-symbolic"
            case ("e0", "01"): # Bluetooth
                icon_name = "xsi-bluetooth-symbolic"
            case ("e0", "02"): # Wi-Fi Adapter
                icon_name = "xsi-network-wireless-symbolic"
            case ("07", _): # Printer
                icon_name = "xsi-printer-symbolic"
            case ("06", _): # Scanner
                icon_name = "xsi-scanner-symbolic"
            case ("0b", _): # Smart Card Reader
                icon_name = "xsi-smartcard-symbolic"
            case ("0d", _): # Security Token
                icon_name = "xsi-smartcard-symbolic"
            case ("ef", _): # Phone / Composite
                icon_name = "xsi-phone-symbolic"
            case ("fe", _): # Firmware Loader
                pass
            case ("ff", _): # Custom Device"
                pass
            case _:
                # Fallback: check interfaces (for composite devices)
                for child in device.children:
                    if child.device_type == 'usb_interface':
                        ic = child.attributes.get('bInterfaceClass').decode().lower()
                        isc = child.attributes.get('bInterfaceSubClass').decode()
                        ip = child.attributes.get('bInterfaceProtocol').decode()
                        if (ic == "00" or isc == "00" or ip == "00"):
                            break
                        # HID boot devices
                        if ic == "03" and isc == "01" and ip == "01": # Keyboard
                            icon_name = "xsi-input-keyboard-symbolic"
                        elif ic == "03" and isc == "01" and ip == "02": # Mouse
                            icon_name = "xsi-input-mouse-symbolic"
                            break # priorize mouse over keyboard
                        elif ic == "0e": # Camera
                            icon_name = "xsi-camera-video-symbolic"
                            break
                        elif ic == "01": # Audio Interface
                            icon_name = "xsi-audio-card-symbolic"
                            break
                        elif ic == "08": # Storage
                            icon_name = "xsi-usb-stick-symbolic"
                            break
    except Exception as e:
        print("Could not find an icon for the device", str(e))

    return icon_name

class USBListWidget(Gtk.ScrolledWindow):
    def __init__(self):
        super().__init__()

        self.context = pyudev.Context()
        self.monitor = pyudev.Monitor.from_netlink(self.context)
        self.monitor.filter_by(subsystem="usb")

        self.treestore = Gtk.TreeStore(
            str, str, str, str, bool, bool, int, str, str, str, bool, bool
        )

        self.treeview = Gtk.TreeView(model=self.treestore)
        self.treeview.connect("row-activated", self.on_row_activated)
        self.treeview.set_enable_tree_lines(True)
        self.treeview.set_property("expand", True)
        self.treeview.set_headers_clickable(True)

        # --- Columns ---
        # Name column with device icon
        icon_renderer = Gtk.CellRendererPixbuf()
        icon_renderer.set_property("xpad", 2)
        icon_renderer.set_property("ypad", 2)
        text_renderer = Gtk.CellRendererText()
        text_renderer.set_property("ypad", 6)
        column = Gtk.TreeViewColumn(_("Name"))
        column.pack_start(icon_renderer, False)
        column.pack_start(text_renderer, True)
        column.add_attribute(icon_renderer, "icon-name", COL_ICON_NAME)
        column.add_attribute(text_renderer, "text", COL_NAME)
        column.add_attribute(text_renderer, "sensitive", COL_SENSITIVE)
        text_renderer.set_property("ellipsize", Pango.EllipsizeMode.END)
        column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
        self.treeview.append_column(column)
        column.set_expand(True)
        column.set_resizable(True)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn(_("ID"), renderer, text=COL_ID)
        column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
        self.treeview.append_column(column)
        column.set_expand(False)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn(_("Speed"), renderer, text=COL_SPEED)
        column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
        self.treeview.append_column(column)
        column.set_expand(False)

        # Power column: symbolic icon + text
        icon_renderer = Gtk.CellRendererPixbuf()
        text_renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn(_("Power"))
        column.pack_start(icon_renderer, False)
        column.pack_start(text_renderer, True)
        column.add_attribute(icon_renderer, "icon-name", COL_POWER_ICON)
        column.add_attribute(text_renderer, "markup", COL_POWER)
        column.add_attribute(text_renderer, "sensitive", COL_SENSITIVE)
        column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
        self.treeview.append_column(column)
        column.set_expand(False)

        self.add(self.treeview)
        self.set_shadow_type(Gtk.ShadowType.IN)

    def load(self):
        self.load_usb_hierarchy()
        threading.Thread(target=self.monitor_usb_events, daemon=True).start()

    def monitor_usb_events(self):
        for _ in self.monitor:
            self.refresh_view()

    def refresh_view(self):
        self.load_usb_hierarchy()
        return False

    @xt.run_idle
    def update_tree(self, rows):
        self.treestore.clear()
        iters = {}
        for row in rows:
            (sysname, parent_name, name, device_id, speed, power_markup, \
            is_controller, mA_val, icon_name, ver_str, power_icon, is_bus_powered, is_hub) = row
            parent_iter = iters.get(parent_name)
            new_iter = self.treestore.append(parent_iter)
            self.treestore.set_value(new_iter, COL_NAME, name)
            self.treestore.set_value(new_iter, COL_ID, device_id)
            self.treestore.set_value(new_iter, COL_SPEED, speed)
            self.treestore.set_value(new_iter, COL_POWER, power_markup)
            self.treestore.set_value(new_iter, COL_SENSITIVE, True)
            self.treestore.set_value(new_iter, COL_IS_CONTROLLER, is_controller)
            self.treestore.set_value(new_iter, COL_MA_VAL, mA_val)
            self.treestore.set_value(new_iter, COL_ICON_NAME, icon_name)
            self.treestore.set_value(new_iter, COL_VERSION, ver_str)
            self.treestore.set_value(new_iter, COL_POWER_ICON, power_icon)
            self.treestore.set_value(new_iter, COL_IS_BUS_POWERED, is_bus_powered)
            self.treestore.set_value(new_iter, COL_IS_HUB, is_hub)
            iters[sysname] = new_iter
        self.evaluate_power_usage()
        root = self.treestore.get_iter_first()
        self.grey_out(root)
        self.treeview.expand_all()

    def load_usb_hierarchy(self):
        devices = {}
        for device in self.context.list_devices(subsystem="usb", DEVTYPE="usb_device"):
            sysname = device.sys_name
            parent = device.find_parent("usb", "usb_device")
            parent_name = None
            if parent is not None:
                parent_name = parent.sys_name
            devices[sysname] = (device, parent_name)

        rows = []

        for sysname, (device, parent_name) in devices.items():
            vid = device.attributes.get("idVendor")
            pid = device.attributes.get("idProduct")
            version = device.attributes.get("version")
            speed = device.attributes.get("speed")
            busnum = device.attributes.get("busnum")
            device_class = device.attributes.get("bDeviceClass")

            vid_str = vid.decode() if vid else ""
            pid_str = pid.decode() if pid else ""
            ver_str = version.decode() if version else ""
            bus_str = busnum.decode() if busnum else "?"
            speed_raw = speed.decode() if speed else ""
            device_class = device_class.decode() if device_class else ""

            speed_pretty = speed_label(speed_raw)
            is_bus_powered, mA_val = decode_power(device)
            icon_name = get_device_icon(device, vid_str)
            is_controller = (vid_str == "1d6b")
            is_hub = (device_class == "09")

            device_id = ""
            speed = ""
            power_markup = ""
            power_icon = ""

            if is_controller:
                name = _("USB %(version)s controller (Bus #%(number)s) – Max speed: %(speed)s") \
                        % {"version": ver_str, 'number': bus_str, 'speed': speed_pretty}
            else:
                vendor = (
                    device.get("ID_VENDOR_FROM_DATABASE")
                    or device.get("ID_VENDOR")
                    or ""
                )
                product = (
                    device.get("ID_MODEL_FROM_DATABASE")
                    or device.get("ID_MODEL")
                    or ""
                )
                name = f"{vendor} {product}".replace("_", " ")
                if vid_str and pid_str:
                    device_id = f"{vid_str}:{pid_str}"
                speed = f"{speed_pretty}"
                if is_bus_powered:
                    power_markup = _("%d mA") % mA_val
                    power_icon = ""
                else:
                    power_markup = _("%d mA") % 0
                    power_icon = ""

            rows.append([sysname, parent_name, name, device_id, speed, power_markup, is_controller, mA_val, icon_name, ver_str, power_icon, is_bus_powered, is_hub])
            self.update_tree(rows)

    # Gray out empty controllers
    def grey_out(self, iter_):
        while iter_:
            is_ctrl = self.treestore.get_value(iter_, COL_IS_CONTROLLER)
            has_child = self.treestore.iter_has_child(iter_)
            if is_ctrl and not has_child:
                self.treestore.set_value(iter_, COL_SENSITIVE, False)
            self.grey_out(self.treestore.iter_children(iter_))
            iter_ = self.treestore.iter_next(iter_)

    # --- Power budget computation ---
    def evaluate_power_usage(self):
        def sum_children(iter_):
            total = 0
            while iter_:
                power_text = self.treestore.get_value(iter_, COL_POWER)
                mA_val = self.treestore.get_value(iter_, COL_MA_VAL)
                is_bus_powered = self.treestore.get_value(iter_, COL_IS_BUS_POWERED)
                is_ctrl = self.treestore.get_value(iter_, COL_IS_CONTROLLER)
                is_hub = self.treestore.get_value(iter_, COL_IS_HUB)
                if is_bus_powered:
                    total += mA_val
                    if is_ctrl or is_hub:
                        total += sum_children(self.treestore.iter_children(iter_))
                elif is_ctrl or is_hub:
                    pass
                else:
                    total += sum_children(self.treestore.iter_children(iter_))
                iter_ = self.treestore.iter_next(iter_)
            return total

        def update_power(iter_):
            while iter_:
                is_ctrl = self.treestore.get_value(iter_, COL_IS_CONTROLLER)
                is_hub = self.treestore.get_value(iter_, COL_IS_HUB)
                if is_ctrl:
                    ver_str = self.treestore.get_value(iter_, COL_VERSION) or "2.00"
                    limit_per_port = usb_limit(ver_str)
                    total_draw = int(sum_children(self.treestore.iter_children(iter_)))
                    max_allowed = limit_per_port * 4
                    percent_draw = int(float(total_draw) / float(max_allowed) * 100.0)
                    value_str = f"{percent_draw}% ({total_draw}/{max_allowed})"
                    text = _("%s mA max") % value_str
                    if total_draw > max_allowed:
                        icon = "dialog-warning-symbolic"
                    else:
                        icon = ""
                    self.treestore.set_value(iter_, COL_POWER, text)
                    self.treestore.set_value(iter_, COL_POWER_ICON, icon)
                update_power(self.treestore.iter_children(iter_))
                iter_ = self.treestore.iter_next(iter_)
        root = self.treestore.get_iter_first()
        update_power(root)

    def on_row_activated(self, treeview, path, column):
        model = treeview.get_model()
        iter_ = model.get_iter(path)
        device_id = model.get_value(iter_, COL_ID)
        if device_id and device_id.strip():
            clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
            clipboard.set_text(device_id, -1)
            subprocess.Popen(['notify-send', '-i', 'xsi-dialog-information-symbolic', _("Device ID copied"), _("The ID of the device was copied into your clipboard.")])

if __name__ == "__main__":
    win = Gtk.Window()
    win.set_default_size(1100, 620)
    win.set_border_width(10)
    widget = USBListWidget()
    win.add(widget)
    win.connect("destroy", Gtk.main_quit)
    win.show_all()
    widget.load()
    Gtk.main()
